# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import sys
from typing import Any, Optional, Union

from azure.core.configuration import Configuration
from azure.core.pipeline import policies

from .. import models as _models

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports

VERSION = "unknown"


class AzureFileStorageConfiguration(Configuration):  # pylint: disable=too-many-instance-attributes
    """Configuration for AzureFileStorage.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param url: The URL of the service account, share, directory or file that is the target of the
     desired operation. Required.
    :type url: str
    :param file_request_intent: Valid value is backup. "backup" Default value is None.
    :type file_request_intent: str or ~azure.storage.fileshare.models.ShareTokenIntent
    :param allow_trailing_dot: If true, the trailing dot will not be trimmed from the target URI.
     Default value is None.
    :type allow_trailing_dot: bool
    :param allow_source_trailing_dot: If true, the trailing dot will not be trimmed from the source
     URI. Default value is None.
    :type allow_source_trailing_dot: bool
    :keyword version: Specifies the version of the operation to use for this request. Default value
     is "2023-01-03". Note that overriding this default value may result in unsupported behavior.
    :paramtype version: str
    :keyword file_range_write_from_url: Only update is supported: - Update: Writes the bytes
     downloaded from the source url into the specified range. Default value is "update". Note that
     overriding this default value may result in unsupported behavior.
    :paramtype file_range_write_from_url: str
    """

    def __init__(
        self,
        url: str,
        file_request_intent: Optional[Union[str, _models.ShareTokenIntent]] = None,
        allow_trailing_dot: Optional[bool] = None,
        allow_source_trailing_dot: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        super(AzureFileStorageConfiguration, self).__init__(**kwargs)
        version: Literal["2023-01-03"] = kwargs.pop("version", "2023-01-03")
        file_range_write_from_url: Literal["update"] = kwargs.pop("file_range_write_from_url", "update")

        if url is None:
            raise ValueError("Parameter 'url' must not be None.")

        self.url = url
        self.file_request_intent = file_request_intent
        self.allow_trailing_dot = allow_trailing_dot
        self.allow_source_trailing_dot = allow_source_trailing_dot
        self.version = version
        self.file_range_write_from_url = file_range_write_from_url
        kwargs.setdefault("sdk_moniker", "azurefilestorage/{}".format(VERSION))
        self._configure(**kwargs)

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get("http_logging_policy") or policies.HttpLoggingPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.AsyncRetryPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get("custom_hook_policy") or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.AsyncRedirectPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, List, Optional, TypeVar, Union

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._service_operations import (
    build_get_properties_request,
    build_list_shares_segment_request,
    build_set_properties_request,
)

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class ServiceOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.storage.fileshare.aio.AzureFileStorage`'s
        :attr:`service` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def set_properties(  # pylint: disable=inconsistent-return-statements
        self, storage_service_properties: _models.StorageServiceProperties, timeout: Optional[int] = None, **kwargs: Any
    ) -> None:
        """Sets properties for a storage account's File service endpoint, including properties for Storage
        Analytics metrics and CORS (Cross-Origin Resource Sharing) rules.

        :param storage_service_properties: The StorageService properties. Required.
        :type storage_service_properties: ~azure.storage.fileshare.models.StorageServiceProperties
        :param timeout: The timeout parameter is expressed in seconds. For more information, see
         :code:`<a
         href="https://docs.microsoft.com/en-us/rest/api/storageservices/Setting-Timeouts-for-File-Service-Operations?redirectedfrom=MSDN">Setting
         Timeouts for File Service Operations.</a>`. Default value is None.
        :type timeout: int
        :keyword restype: restype. Default value is "service". Note that overriding this default value
         may result in unsupported behavior.
        :paramtype restype: str
        :keyword comp: comp. Default value is "properties". Note that overriding this default value may
         result in unsupported behavior.
        :paramtype comp: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        restype: Literal["service"] = kwargs.pop("restype", _params.pop("restype", "service"))
        comp: Literal["properties"] = kwargs.pop("comp", _params.pop("comp", "properties"))
        content_type: str = kwargs.pop("content_type", _headers.pop("Content-Type", "application/xml"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = self._serialize.body(storage_service_properties, "StorageServiceProperties", is_xml=True)

        request = build_set_properties_request(
            url=self._config.url,
            timeout=timeout,
            restype=restype,
            comp=comp,
            content_type=content_type,
            version=self._config.version,
            content=_content,
            template_url=self.set_properties.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.StorageError, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["x-ms-request-id"] = self._deserialize("str", response.headers.get("x-ms-request-id"))
        response_headers["x-ms-version"] = self._deserialize("str", response.headers.get("x-ms-version"))

        if cls:
            return cls(pipeline_response, None, response_headers)

    set_properties.metadata = {"url": "{url}"}

    @distributed_trace_async
    async def get_properties(self, timeout: Optional[int] = None, **kwargs: Any) -> _models.StorageServiceProperties:
        """Gets the properties of a storage account's File service, including properties for Storage
        Analytics metrics and CORS (Cross-Origin Resource Sharing) rules.

        :param timeout: The timeout parameter is expressed in seconds. For more information, see
         :code:`<a
         href="https://docs.microsoft.com/en-us/rest/api/storageservices/Setting-Timeouts-for-File-Service-Operations?redirectedfrom=MSDN">Setting
         Timeouts for File Service Operations.</a>`. Default value is None.
        :type timeout: int
        :keyword restype: restype. Default value is "service". Note that overriding this default value
         may result in unsupported behavior.
        :paramtype restype: str
        :keyword comp: comp. Default value is "properties". Note that overriding this default value may
         result in unsupported behavior.
        :paramtype comp: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: StorageServiceProperties or the result of cls(response)
        :rtype: ~azure.storage.fileshare.models.StorageServiceProperties
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        restype: Literal["service"] = kwargs.pop("restype", _params.pop("restype", "service"))
        comp: Literal["properties"] = kwargs.pop("comp", _params.pop("comp", "properties"))
        cls: ClsType[_models.StorageServiceProperties] = kwargs.pop("cls", None)

        request = build_get_properties_request(
            url=self._config.url,
            timeout=timeout,
            restype=restype,
            comp=comp,
            version=self._config.version,
            template_url=self.get_properties.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.StorageError, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["x-ms-request-id"] = self._deserialize("str", response.headers.get("x-ms-request-id"))
        response_headers["x-ms-version"] = self._deserialize("str", response.headers.get("x-ms-version"))

        deserialized = self._deserialize("StorageServiceProperties", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    get_properties.metadata = {"url": "{url}"}

    @distributed_trace_async
    async def list_shares_segment(
        self,
        prefix: Optional[str] = None,
        marker: Optional[str] = None,
        maxresults: Optional[int] = None,
        include: Optional[List[Union[str, _models.ListSharesIncludeType]]] = None,
        timeout: Optional[int] = None,
        **kwargs: Any
    ) -> _models.ListSharesResponse:
        """The List Shares Segment operation returns a list of the shares and share snapshots under the
        specified account.

        :param prefix: Filters the results to return only entries whose name begins with the specified
         prefix. Default value is None.
        :type prefix: str
        :param marker: A string value that identifies the portion of the list to be returned with the
         next list operation. The operation returns a marker value within the response body if the list
         returned was not complete. The marker value may then be used in a subsequent call to request
         the next set of list items. The marker value is opaque to the client. Default value is None.
        :type marker: str
        :param maxresults: Specifies the maximum number of entries to return. If the request does not
         specify maxresults, or specifies a value greater than 5,000, the server will return up to 5,000
         items. Default value is None.
        :type maxresults: int
        :param include: Include this parameter to specify one or more datasets to include in the
         response. Default value is None.
        :type include: list[str or ~azure.storage.fileshare.models.ListSharesIncludeType]
        :param timeout: The timeout parameter is expressed in seconds. For more information, see
         :code:`<a
         href="https://docs.microsoft.com/en-us/rest/api/storageservices/Setting-Timeouts-for-File-Service-Operations?redirectedfrom=MSDN">Setting
         Timeouts for File Service Operations.</a>`. Default value is None.
        :type timeout: int
        :keyword comp: comp. Default value is "list". Note that overriding this default value may
         result in unsupported behavior.
        :paramtype comp: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: ListSharesResponse or the result of cls(response)
        :rtype: ~azure.storage.fileshare.models.ListSharesResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        comp: Literal["list"] = kwargs.pop("comp", _params.pop("comp", "list"))
        cls: ClsType[_models.ListSharesResponse] = kwargs.pop("cls", None)

        request = build_list_shares_segment_request(
            url=self._config.url,
            prefix=prefix,
            marker=marker,
            maxresults=maxresults,
            include=include,
            timeout=timeout,
            comp=comp,
            version=self._config.version,
            template_url=self.list_shares_segment.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.StorageError, pipeline_response)
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["x-ms-request-id"] = self._deserialize("str", response.headers.get("x-ms-request-id"))
        response_headers["x-ms-version"] = self._deserialize("str", response.headers.get("x-ms-version"))

        deserialized = self._deserialize("ListSharesResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    list_shares_segment.metadata = {"url": "{url}"}



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from ._test_base import _FileTest

from azure_devtools.perfstress_tests import RandomStream, get_random_bytes
from azure_devtools.perfstress_tests import AsyncRandomStream


class UploadTest(_FileTest):
    def __init__(self, arguments):
        super().__init__(arguments)
        self.upload_stream = RandomStream(self.args.size)
        self.upload_stream_async = AsyncRandomStream(self.args.size)

    def run_sync(self):
        self.upload_stream.reset()
        self.sharefile_client.upload_file(
            self.upload_stream,
            length=self.args.size,
            max_concurrency=self.args.max_concurrency)

    async def run_async(self):
        self.upload_stream_async.reset()
        await self.async_sharefile_client.upload_file(
            self.upload_stream_async,
            length=self.args.size,
            max_concurrency=self.args.max_concurrency)



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
import platform

import pytest

from devtools_testutils import (
    add_body_regex_sanitizer,
    add_general_regex_sanitizer,
    add_header_regex_sanitizer,
    add_oauth_response_sanitizer,
    add_uri_string_sanitizer,
    test_proxy
)

@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy):
    subscription_id = os.environ.get("AZURE_SUBSCRIPTION_ID", "00000000-0000-0000-0000-000000000000")
    tenant_id = os.environ.get("STORAGE_TENANT_ID", "00000000-0000-0000-0000-000000000000")
    add_general_regex_sanitizer(regex=subscription_id, value="00000000-0000-0000-0000-000000000000")
    add_general_regex_sanitizer(regex=tenant_id, value="00000000-0000-0000-0000-000000000000")
    add_header_regex_sanitizer(key="Set-Cookie", value="[set-cookie;]")
    add_header_regex_sanitizer(key="Cookie", value="cookie;")
    add_oauth_response_sanitizer()

    add_header_regex_sanitizer(key="x-ms-copy-source-authorization", value="Sanitized")
    add_header_regex_sanitizer(key="x-ms-encryption-key", value="Sanitized")
    add_body_regex_sanitizer(regex=r"<ClientIp>.*?</ClientIp>", value="<ClientIp>0.0.0.0:0</ClientIp>")

    add_uri_string_sanitizer(target=".preprod.", value=".")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import tempfile

from azure_devtools.perfstress_tests import get_random_bytes

from ._test_base import _FileTest


class UploadFromFileTest(_FileTest):
    temp_file = None

    async def global_setup(self):
        await super().global_setup()
        data = get_random_bytes(self.args.size)
        with tempfile.NamedTemporaryFile(delete=False) as temp_file:
            UploadFromFileTest.temp_file = temp_file.name
            temp_file.write(data)

    async def global_cleanup(self):
        if self.temp_file:
            os.remove(UploadFromFileTest.temp_file)
        await super().global_cleanup()

    def run_sync(self):
        with open(UploadFromFileTest.temp_file, 'rb') as fp:
            self.sharefile_client.upload_file(fp, max_concurrency=self.args.max_concurrency)

    async def run_async(self):
        with open(UploadFromFileTest.temp_file, 'rb') as fp:
            await self.async_sharefile_client.upload_file(fp, max_concurrency=self.args.max_concurrency)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import uuid

from azure_devtools.perfstress_tests import PerfStressTest

from azure.storage.file import FileService

class _LegacyServiceTest(PerfStressTest):
    service_client = None
    async_service_client = None

    def __init__(self, arguments):
        super().__init__(arguments)
        connection_string = self.get_from_env("AZURE_STORAGE_CONNECTION_STRING")
        if not _LegacyServiceTest.service_client or self.args.no_client_share:
            _LegacyServiceTest.service_client = FileService(connection_string=connection_string)
            if self.args.max_range_size:
                _LegacyServiceTest.service_client.MAX_RANGE_SIZE = self.args.max_range_size
        self.async_service_client = None
        self.service_client = _LegacyServiceTest.service_client

    @staticmethod
    def add_arguments(parser):
        super(_LegacyServiceTest, _LegacyServiceTest).add_arguments(parser)
        parser.add_argument('-r', '--max-range-size', nargs='?', type=int, help='Maximum size of data uploading in single HTTP PUT. Defaults to 4*1024*1024', default=4*1024*1024)
        parser.add_argument('-c', '--max-concurrency', nargs='?', type=int, help='Maximum number of concurrent threads used for data transfer. Defaults to 1', default=1)
        parser.add_argument('-s', '--size', nargs='?', type=int, help='Size of data to transfer.  Default is 10240.', default=10240)
        parser.add_argument('--no-client-share', action='store_true', help='Create one ServiceClient per test instance.  Default is to share a single ServiceClient.', default=False)


class _LegacyShareTest(_LegacyServiceTest):
    share_name = "perfstress-legacy-" + str(uuid.uuid4())

    def __init__(self, arguments):
        super().__init__(arguments)

    async def global_setup(self):
        await super().global_setup()
        self.service_client.create_share(self.share_name)

    async def global_cleanup(self):
        self.service_client.delete_share(self.share_name)
        await super().global_cleanup()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

from ._file_client_async import ShareFileClient
from ._directory_client_async import ShareDirectoryClient
from ._share_client_async import ShareClient
from ._share_service_client_async import ShareServiceClient
from ._lease_async import ShareLeaseClient


__all__ = [
    'ShareFileClient',
    'ShareDirectoryClient',
    'ShareClient',
    'ShareServiceClient',
    'ShareLeaseClient',
]



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import unittest

import pytest
from azure.storage.fileshare.aio import ShareServiceClient

from devtools_testutils.aio import recorded_by_proxy_async
from devtools_testutils.storage.aio import AsyncStorageRecordedTestCase
from settings.testcase import FileSharePreparer

# ------------------------------------------------------------------------------
TEST_SHARE_NAME = 'test-share'
# ------------------------------------------------------------------------------


class TestStorageHandleAsync(AsyncStorageRecordedTestCase):
    def _setup(self, storage_account, storage_account_key):
        file_url = self.account_url(storage_account, "file")
        credentials = storage_account_key
        self.fsc = ShareServiceClient(account_url=file_url, credential=credentials)

    # --Helpers-----------------------------------------------------------------

    def _validate_handles(self, handles):
        # Assert
        assert handles is not None
        assert len(handles) >= 1
        assert handles[0] is not None

        # verify basic fields
        # path may or may not be present
        # last_connect_time_string has been missing in the test
        assert handles[0].id is not None
        assert handles[0].file_id is not None
        assert handles[0].parent_id is not None
        assert handles[0].session_id is not None
        assert handles[0].client_ip is not None
        assert handles[0].open_time is not None

    @pytest.mark.playback_test_only
    @FileSharePreparer()
    @recorded_by_proxy_async
    async def test_close_single_handle(self, **kwargs):
        storage_account_name = kwargs.pop("storage_account_name")
        storage_account_key = kwargs.pop("storage_account_key")

        # don't run live, since the test set up was highly manual
        # only run when recording, or playing back in CI

        self._setup(storage_account_name, storage_account_key)
        share = self.fsc.get_share_client(TEST_SHARE_NAME)
        root = share.get_directory_client()
        handles = []
        async for handle in root.list_handles(recursive=True):
            handles.append(handle)
        self._validate_handles(handles)

        # Act
        with pytest.raises(ValueError):
            await root.close_handle('*')

        handles_info = await root.close_handle(handles[0])

        # Assert 1 handle has been closed
        assert 1 == handles_info['closed_handles_count']
        assert handles_info['failed_handles_count'] == 0

    @pytest.mark.playback_test_only
    @FileSharePreparer()
    @recorded_by_proxy_async
    async def test_close_all_handle(self, **kwargs):
        storage_account_name = kwargs.pop("storage_account_name")
        storage_account_key = kwargs.pop("storage_account_key")

        # don't run live, since the test set up was highly manual
        # only run when recording, or playing back in CI

        self._setup(storage_account_name, storage_account_key)
        share = self.fsc.get_share_client(TEST_SHARE_NAME)
        root = share.get_directory_client()
        handles = []
        async for handle in root.list_handles(recursive=True):
            handles.append(handle)
        self._validate_handles(handles)

        # Act
        handles_info = await root.close_all_handles(recursive=True)

        # Assert at least 1 handle has been closed
        assert handles_info['closed_handles_count'] > 1
        assert handles_info['failed_handles_count'] == 0

    @pytest.mark.playback_test_only
    @FileSharePreparer()
    @recorded_by_proxy_async
    async def test_list_handles_access_rights(self, **kwargs):
        storage_account_name = kwargs.pop("storage_account_name")
        storage_account_key = kwargs.pop("storage_account_key")

        # don't run live, since the test set up was highly manual
        # only run when recording, or playing back in CI

        self._setup(storage_account_name, storage_account_key)
        share = self.fsc.get_share_client('testshare')
        root = share.get_directory_client('testdir')
        file_client = root.get_file_client('testfile.txt')

        # Act
        handles = []
        async for handle in file_client.list_handles():
            handles.append(handle)

        # Assert
        self._validate_handles(handles)
        handles[0]['access_rights'][0] == 'Write'


# ------------------------------------------------------------------------------
if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: file_samples_share_async.py

DESCRIPTION:
    These samples demonstrate share operations like creating a share snapshot,
    setting share quota and metadata, listing directories and files in the
    file share, and getting directory and file clients from a share client.

USAGE:
    python file_samples_share_async.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_STORAGE_CONNECTION_STRING - the connection string to your storage account
"""

import os
import asyncio
from azure.storage.fileshare import ShareAccessTier

SOURCE_FILE = './SampleSource.txt'
DEST_FILE = './SampleDestination.txt'


class ShareSamplesAsync(object):

    connection_string = os.getenv('AZURE_STORAGE_CONNECTION_STRING')

    async def create_share_snapshot_async(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare.aio import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples1")

        async with share:
            # [START create_share]
            # Create share with Access Tier set to Hot
            await share.create_share(access_tier=ShareAccessTier("Hot"))
            # [END create_share]
            try:
                # [START create_share_snapshot]
                await share.create_snapshot()
                # [END create_share_snapshot]
            finally:
                # [START delete_share]
                await share.delete_share(delete_snapshots=True)
                # [END delete_share]

    async def set_share_quota_and_metadata_async(self):
        # [START create_share_client_from_conn_string]
        from azure.storage.fileshare.aio import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples2")
        # [END create_share_client_from_conn_string]

        # Create the share
        async with share:
            await share.create_share()

            try:
                # [START set_share_quota]
                # Set the quota for the share to 1GB
                await share.set_share_quota(quota=1)
                # [END set_share_quota]

                # [START set_share_metadata]
                data = {'category': 'test'}
                await share.set_share_metadata(metadata=data)
                # [END set_share_metadata]

                # Get the metadata for the share
                props = (await share.get_share_properties()).metadata

            finally:
                # Delete the share
                await share.delete_share()

    async def set_share_properties(self):
        from azure.storage.fileshare.aio import ShareClient
        share1 = ShareClient.from_connection_string(self.connection_string, "sharesamples3a")
        share2 = ShareClient.from_connection_string(self.connection_string, "sharesamples3b")

        # Create the share
        async with share1 and share2:
            await share1.create_share()
            await share2.create_share()

            try:
                # [START set_share_properties]
                # Set the tier for the first share to Hot
                await share1.set_share_properties(access_tier="Hot")
                # Set the quota for the first share to 3
                await share1.set_share_properties(quota=3)
                # Set the tier for the second share to Cool and quota to 2
                await share2.set_share_properties(access_tier=ShareAccessTier("Cool"), quota=2)

                # Get the shares' properties
                props1 = await share1.get_share_properties()
                props2 = await share2.get_share_properties()
                print(props1.access_tier)
                print(props1.quota)
                print(props2.access_tier)
                print(props2.quota)
                # [END set_share_properties]

            finally:
                # Delete the shares
                await share1.delete_share()
                await share2.delete_share()

    async def list_directories_and_files_async(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare.aio import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples4")

        # Create the share
        async with share:
            await share.create_share()

            try:
                # [START share_list_files_in_dir]
                # Create a directory in the share
                dir_client = await share.create_directory("mydir")

                # Upload a file to the directory
                with open(SOURCE_FILE, "rb") as source_file:
                    await dir_client.upload_file(file_name="sample", data=source_file)

                # List files in the directory
                my_files = []
                async for item in share.list_directories_and_files(directory_name="mydir"):
                    my_files.append(item)
                print(my_files)
                # [END share_list_files_in_dir]
            finally:
                # Delete the share
                await share.delete_share()

    async def get_directory_or_file_client_async(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare.aio import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples5")

        # Get the directory client to interact with a specific directory
        my_dir = share.get_directory_client("dir1")

        # Get the file client to interact with a specific file
        my_file = share.get_file_client("dir1/myfile")


async def main():
    sample = ShareSamplesAsync()
    await sample.create_share_snapshot_async()
    await sample.set_share_quota_and_metadata_async()
    await sample.set_share_properties()
    await sample.list_directories_and_files_async()
    await sample.get_directory_or_file_client_async()

if __name__ == '__main__':
    asyncio.run(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AccessPolicy
from ._models_py3 import ClearRange
from ._models_py3 import CopyFileSmbInfo
from ._models_py3 import CorsRule
from ._models_py3 import DestinationLeaseAccessConditions
from ._models_py3 import DirectoryItem
from ._models_py3 import FileHTTPHeaders
from ._models_py3 import FileItem
from ._models_py3 import FileProperty
from ._models_py3 import FileRange
from ._models_py3 import FilesAndDirectoriesListSegment
from ._models_py3 import HandleItem
from ._models_py3 import LeaseAccessConditions
from ._models_py3 import ListFilesAndDirectoriesSegmentResponse
from ._models_py3 import ListHandlesResponse
from ._models_py3 import ListSharesResponse
from ._models_py3 import Metrics
from ._models_py3 import RetentionPolicy
from ._models_py3 import ShareFileRangeList
from ._models_py3 import ShareItemInternal
from ._models_py3 import SharePermission
from ._models_py3 import SharePropertiesInternal
from ._models_py3 import ShareProtocolSettings
from ._models_py3 import ShareSmbSettings
from ._models_py3 import ShareStats
from ._models_py3 import SignedIdentifier
from ._models_py3 import SmbMultichannel
from ._models_py3 import SourceLeaseAccessConditions
from ._models_py3 import SourceModifiedAccessConditions
from ._models_py3 import StorageError
from ._models_py3 import StorageServiceProperties
from ._models_py3 import StringEncoded

from ._azure_file_storage_enums import AccessRight
from ._azure_file_storage_enums import CopyStatusType
from ._azure_file_storage_enums import DeleteSnapshotsOptionType
from ._azure_file_storage_enums import FileLastWrittenMode
from ._azure_file_storage_enums import FileRangeWriteType
from ._azure_file_storage_enums import LeaseDurationType
from ._azure_file_storage_enums import LeaseStateType
from ._azure_file_storage_enums import LeaseStatusType
from ._azure_file_storage_enums import ListFilesIncludeType
from ._azure_file_storage_enums import ListSharesIncludeType
from ._azure_file_storage_enums import PermissionCopyModeType
from ._azure_file_storage_enums import ShareAccessTier
from ._azure_file_storage_enums import ShareRootSquash
from ._azure_file_storage_enums import ShareTokenIntent
from ._azure_file_storage_enums import StorageErrorCode
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AccessPolicy",
    "ClearRange",
    "CopyFileSmbInfo",
    "CorsRule",
    "DestinationLeaseAccessConditions",
    "DirectoryItem",
    "FileHTTPHeaders",
    "FileItem",
    "FileProperty",
    "FileRange",
    "FilesAndDirectoriesListSegment",
    "HandleItem",
    "LeaseAccessConditions",
    "ListFilesAndDirectoriesSegmentResponse",
    "ListHandlesResponse",
    "ListSharesResponse",
    "Metrics",
    "RetentionPolicy",
    "ShareFileRangeList",
    "ShareItemInternal",
    "SharePermission",
    "SharePropertiesInternal",
    "ShareProtocolSettings",
    "ShareSmbSettings",
    "ShareStats",
    "SignedIdentifier",
    "SmbMultichannel",
    "SourceLeaseAccessConditions",
    "SourceModifiedAccessConditions",
    "StorageError",
    "StorageServiceProperties",
    "StringEncoded",
    "AccessRight",
    "CopyStatusType",
    "DeleteSnapshotsOptionType",
    "FileLastWrittenMode",
    "FileRangeWriteType",
    "LeaseDurationType",
    "LeaseStateType",
    "LeaseStatusType",
    "ListFilesIncludeType",
    "ListSharesIncludeType",
    "PermissionCopyModeType",
    "ShareAccessTier",
    "ShareRootSquash",
    "ShareTokenIntent",
    "StorageErrorCode",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

STORAGE_ACCOUNT_NAME = "fakename"
STORAGE_ACCOUNT_KEY = "fakekey"
PREMIUM_STORAGE_FILE_ACCOUNT_NAME = "fakename"
PREMIUM_STORAGE_FILE_ACCOUNT_KEY = "fakekey"
SECONDARY_STORAGE_ACCOUNT_NAME = "fakename"
SECONDARY_STORAGE_ACCOUNT_KEY = "fakekey"

TENANT_ID = "00000000-0000-0000-0000-000000000000"
CLIENT_ID = "00000000-0000-0000-0000-000000000000"
CLIENT_SECRET = "00000000-0000-0000-0000-000000000000"

ACCOUNT_URL_SUFFIX = 'core.windows.net'
RUN_IN_LIVE = "False"
SKIP_LIVE_RECORDING = "True"

PROTOCOL = "https"



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure_devtools.perfstress_tests import get_random_bytes, WriteStream

from ._test_base import _ShareTest


class DownloadTest(_ShareTest):
    def __init__(self, arguments):
        super().__init__(arguments)
        file_name = "downloadtest"
        self.sharefile_client = self.share_client.get_file_client(file_name)
        self.async_sharefile_client = self.async_share_client.get_file_client(file_name)
        self.download_stream = WriteStream()

    async def global_setup(self):
        await super().global_setup()
        data = get_random_bytes(self.args.size)
        await self.async_sharefile_client.upload_file(data)

    def run_sync(self):
        self.download_stream.reset()
        stream = self.sharefile_client.download_file(max_concurrency=self.args.max_concurrency)
        stream.readinto(self.download_stream)

    async def run_async(self):
        self.download_stream.reset()
        stream = await self.async_sharefile_client.download_file(max_concurrency=self.args.max_concurrency)
        await stream.readinto(self.download_stream)

    async def close(self):
        await self.async_sharefile_client.close()
        await super().close()



# coding=utf-8
# pylint: disable=too-many-lines
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Any, Dict, List, Optional, TYPE_CHECKING, Union

from .. import _serialization

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from .. import models as _models


class AccessPolicy(_serialization.Model):
    """An Access policy.

    :ivar start: The date-time the policy is active.
    :vartype start: str
    :ivar expiry: The date-time the policy expires.
    :vartype expiry: str
    :ivar permission: The permissions for the ACL policy.
    :vartype permission: str
    """

    _attribute_map = {
        "start": {"key": "Start", "type": "str"},
        "expiry": {"key": "Expiry", "type": "str"},
        "permission": {"key": "Permission", "type": "str"},
    }

    def __init__(
        self,
        *,
        start: Optional[str] = None,
        expiry: Optional[str] = None,
        permission: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword start: The date-time the policy is active.
        :paramtype start: str
        :keyword expiry: The date-time the policy expires.
        :paramtype expiry: str
        :keyword permission: The permissions for the ACL policy.
        :paramtype permission: str
        """
        super().__init__(**kwargs)
        self.start = start
        self.expiry = expiry
        self.permission = permission


class ClearRange(_serialization.Model):
    """ClearRange.

    All required parameters must be populated in order to send to Azure.

    :ivar start: Required.
    :vartype start: int
    :ivar end: Required.
    :vartype end: int
    """

    _validation = {
        "start": {"required": True},
        "end": {"required": True},
    }

    _attribute_map = {
        "start": {"key": "Start", "type": "int", "xml": {"name": "Start"}},
        "end": {"key": "End", "type": "int", "xml": {"name": "End"}},
    }
    _xml_map = {"name": "ClearRange"}

    def __init__(self, *, start: int, end: int, **kwargs: Any) -> None:
        """
        :keyword start: Required.
        :paramtype start: int
        :keyword end: Required.
        :paramtype end: int
        """
        super().__init__(**kwargs)
        self.start = start
        self.end = end


class CopyFileSmbInfo(_serialization.Model):
    """Parameter group.

    :ivar file_attributes: Specifies either the option to copy file attributes from a source
     file(source) to a target file or a list of attributes to set on a target file.
    :vartype file_attributes: str
    :ivar file_creation_time: Specifies either the option to copy file creation time from a source
     file(source) to a target file or a time value in ISO 8601 format to set as creation time on a
     target file.
    :vartype file_creation_time: str
    :ivar file_last_write_time: Specifies either the option to copy file last write time from a
     source file(source) to a target file or a time value in ISO 8601 format to set as last write
     time on a target file.
    :vartype file_last_write_time: str
    :ivar file_change_time: Specifies either the option to copy file last write time from a source
     file(source) to a target file or a time value in ISO 8601 format to set as last write time on a
     target file.
    :vartype file_change_time: str
    :ivar file_permission_copy_mode: Specifies the option to copy file security descriptor from
     source file or to set it using the value which is defined by the header value of
     x-ms-file-permission or x-ms-file-permission-key. Known values are: "source" and "override".
    :vartype file_permission_copy_mode: str or
     ~azure.storage.fileshare.models.PermissionCopyModeType
    :ivar ignore_read_only: Specifies the option to overwrite the target file if it already exists
     and has read-only attribute set.
    :vartype ignore_read_only: bool
    :ivar set_archive_attribute: Specifies the option to set archive attribute on a target file.
     True means archive attribute will be set on a target file despite attribute overrides or a
     source file state.
    :vartype set_archive_attribute: bool
    """

    _attribute_map = {
        "file_attributes": {"key": "fileAttributes", "type": "str"},
        "file_creation_time": {"key": "fileCreationTime", "type": "str"},
        "file_last_write_time": {"key": "fileLastWriteTime", "type": "str"},
        "file_change_time": {"key": "fileChangeTime", "type": "str"},
        "file_permission_copy_mode": {"key": "filePermissionCopyMode", "type": "str"},
        "ignore_read_only": {"key": "ignoreReadOnly", "type": "bool"},
        "set_archive_attribute": {"key": "setArchiveAttribute", "type": "bool"},
    }

    def __init__(
        self,
        *,
        file_attributes: Optional[str] = None,
        file_creation_time: Optional[str] = None,
        file_last_write_time: Optional[str] = None,
        file_change_time: Optional[str] = None,
        file_permission_copy_mode: Optional[Union[str, "_models.PermissionCopyModeType"]] = None,
        ignore_read_only: Optional[bool] = None,
        set_archive_attribute: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword file_attributes: Specifies either the option to copy file attributes from a source
         file(source) to a target file or a list of attributes to set on a target file.
        :paramtype file_attributes: str
        :keyword file_creation_time: Specifies either the option to copy file creation time from a
         source file(source) to a target file or a time value in ISO 8601 format to set as creation time
         on a target file.
        :paramtype file_creation_time: str
        :keyword file_last_write_time: Specifies either the option to copy file last write time from a
         source file(source) to a target file or a time value in ISO 8601 format to set as last write
         time on a target file.
        :paramtype file_last_write_time: str
        :keyword file_change_time: Specifies either the option to copy file last write time from a
         source file(source) to a target file or a time value in ISO 8601 format to set as last write
         time on a target file.
        :paramtype file_change_time: str
        :keyword file_permission_copy_mode: Specifies the option to copy file security descriptor from
         source file or to set it using the value which is defined by the header value of
         x-ms-file-permission or x-ms-file-permission-key. Known values are: "source" and "override".
        :paramtype file_permission_copy_mode: str or
         ~azure.storage.fileshare.models.PermissionCopyModeType
        :keyword ignore_read_only: Specifies the option to overwrite the target file if it already
         exists and has read-only attribute set.
        :paramtype ignore_read_only: bool
        :keyword set_archive_attribute: Specifies the option to set archive attribute on a target file.
         True means archive attribute will be set on a target file despite attribute overrides or a
         source file state.
        :paramtype set_archive_attribute: bool
        """
        super().__init__(**kwargs)
        self.file_attributes = file_attributes
        self.file_creation_time = file_creation_time
        self.file_last_write_time = file_last_write_time
        self.file_change_time = file_change_time
        self.file_permission_copy_mode = file_permission_copy_mode
        self.ignore_read_only = ignore_read_only
        self.set_archive_attribute = set_archive_attribute


class CorsRule(_serialization.Model):
    """CORS is an HTTP feature that enables a web application running under one domain to access
    resources in another domain. Web browsers implement a security restriction known as same-origin
    policy that prevents a web page from calling APIs in a different domain; CORS provides a secure
    way to allow one domain (the origin domain) to call APIs in another domain.

    All required parameters must be populated in order to send to Azure.

    :ivar allowed_origins: The origin domains that are permitted to make a request against the
     storage service via CORS. The origin domain is the domain from which the request originates.
     Note that the origin must be an exact case-sensitive match with the origin that the user age
     sends to the service. You can also use the wildcard character '*' to allow all origin domains
     to make requests via CORS. Required.
    :vartype allowed_origins: str
    :ivar allowed_methods: The methods (HTTP request verbs) that the origin domain may use for a
     CORS request. (comma separated). Required.
    :vartype allowed_methods: str
    :ivar allowed_headers: The request headers that the origin domain may specify on the CORS
     request. Required.
    :vartype allowed_headers: str
    :ivar exposed_headers: The response headers that may be sent in the response to the CORS
     request and exposed by the browser to the request issuer. Required.
    :vartype exposed_headers: str
    :ivar max_age_in_seconds: The maximum amount time that a browser should cache the preflight
     OPTIONS request. Required.
    :vartype max_age_in_seconds: int
    """

    _validation = {
        "allowed_origins": {"required": True},
        "allowed_methods": {"required": True},
        "allowed_headers": {"required": True},
        "exposed_headers": {"required": True},
        "max_age_in_seconds": {"required": True, "minimum": 0},
    }

    _attribute_map = {
        "allowed_origins": {"key": "AllowedOrigins", "type": "str"},
        "allowed_methods": {"key": "AllowedMethods", "type": "str"},
        "allowed_headers": {"key": "AllowedHeaders", "type": "str"},
        "exposed_headers": {"key": "ExposedHeaders", "type": "str"},
        "max_age_in_seconds": {"key": "MaxAgeInSeconds", "type": "int"},
    }

    def __init__(
        self,
        *,
        allowed_origins: str,
        allowed_methods: str,
        allowed_headers: str,
        exposed_headers: str,
        max_age_in_seconds: int,
        **kwargs: Any
    ) -> None:
        """
        :keyword allowed_origins: The origin domains that are permitted to make a request against the
         storage service via CORS. The origin domain is the domain from which the request originates.
         Note that the origin must be an exact case-sensitive match with the origin that the user age
         sends to the service. You can also use the wildcard character '*' to allow all origin domains
         to make requests via CORS. Required.
        :paramtype allowed_origins: str
        :keyword allowed_methods: The methods (HTTP request verbs) that the origin domain may use for a
         CORS request. (comma separated). Required.
        :paramtype allowed_methods: str
        :keyword allowed_headers: The request headers that the origin domain may specify on the CORS
         request. Required.
        :paramtype allowed_headers: str
        :keyword exposed_headers: The response headers that may be sent in the response to the CORS
         request and exposed by the browser to the request issuer. Required.
        :paramtype exposed_headers: str
        :keyword max_age_in_seconds: The maximum amount time that a browser should cache the preflight
         OPTIONS request. Required.
        :paramtype max_age_in_seconds: int
        """
        super().__init__(**kwargs)
        self.allowed_origins = allowed_origins
        self.allowed_methods = allowed_methods
        self.allowed_headers = allowed_headers
        self.exposed_headers = exposed_headers
        self.max_age_in_seconds = max_age_in_seconds


class DestinationLeaseAccessConditions(_serialization.Model):
    """Parameter group.

    :ivar destination_lease_id: Required if the destination file has an active infinite lease. The
     lease ID specified for this header must match the lease ID of the destination file. If the
     request does not include the lease ID or it is not valid, the operation fails with status code
     412 (Precondition Failed). If this header is specified and the destination file does not
     currently have an active lease, the operation will also fail with status code 412 (Precondition
     Failed).
    :vartype destination_lease_id: str
    """

    _attribute_map = {
        "destination_lease_id": {"key": "destinationLeaseId", "type": "str"},
    }

    def __init__(self, *, destination_lease_id: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword destination_lease_id: Required if the destination file has an active infinite lease.
         The lease ID specified for this header must match the lease ID of the destination file. If the
         request does not include the lease ID or it is not valid, the operation fails with status code
         412 (Precondition Failed). If this header is specified and the destination file does not
         currently have an active lease, the operation will also fail with status code 412 (Precondition
         Failed).
        :paramtype destination_lease_id: str
        """
        super().__init__(**kwargs)
        self.destination_lease_id = destination_lease_id


class DirectoryItem(_serialization.Model):
    """A listed directory item.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required.
    :vartype name: ~azure.storage.fileshare.models.StringEncoded
    :ivar file_id:
    :vartype file_id: str
    :ivar properties: File properties.
    :vartype properties: ~azure.storage.fileshare.models.FileProperty
    :ivar attributes:
    :vartype attributes: str
    :ivar permission_key:
    :vartype permission_key: str
    """

    _validation = {
        "name": {"required": True},
    }

    _attribute_map = {
        "name": {"key": "Name", "type": "StringEncoded"},
        "file_id": {"key": "FileId", "type": "str"},
        "properties": {"key": "Properties", "type": "FileProperty"},
        "attributes": {"key": "Attributes", "type": "str"},
        "permission_key": {"key": "PermissionKey", "type": "str"},
    }
    _xml_map = {"name": "Directory"}

    def __init__(
        self,
        *,
        name: "_models.StringEncoded",
        file_id: Optional[str] = None,
        properties: Optional["_models.FileProperty"] = None,
        attributes: Optional[str] = None,
        permission_key: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: Required.
        :paramtype name: ~azure.storage.fileshare.models.StringEncoded
        :keyword file_id:
        :paramtype file_id: str
        :keyword properties: File properties.
        :paramtype properties: ~azure.storage.fileshare.models.FileProperty
        :keyword attributes:
        :paramtype attributes: str
        :keyword permission_key:
        :paramtype permission_key: str
        """
        super().__init__(**kwargs)
        self.name = name
        self.file_id = file_id
        self.properties = properties
        self.attributes = attributes
        self.permission_key = permission_key


class FileHTTPHeaders(_serialization.Model):
    """Parameter group.

    :ivar file_content_type: Sets the MIME content type of the file. The default type is
     'application/octet-stream'.
    :vartype file_content_type: str
    :ivar file_content_encoding: Specifies which content encodings have been applied to the file.
    :vartype file_content_encoding: str
    :ivar file_content_language: Specifies the natural languages used by this resource.
    :vartype file_content_language: str
    :ivar file_cache_control: Sets the file's cache control. The File service stores this value but
     does not use or modify it.
    :vartype file_cache_control: str
    :ivar file_content_md5: Sets the file's MD5 hash.
    :vartype file_content_md5: bytes
    :ivar file_content_disposition: Sets the file's Content-Disposition header.
    :vartype file_content_disposition: str
    """

    _attribute_map = {
        "file_content_type": {"key": "fileContentType", "type": "str"},
        "file_content_encoding": {"key": "fileContentEncoding", "type": "str"},
        "file_content_language": {"key": "fileContentLanguage", "type": "str"},
        "file_cache_control": {"key": "fileCacheControl", "type": "str"},
        "file_content_md5": {"key": "fileContentMD5", "type": "bytearray"},
        "file_content_disposition": {"key": "fileContentDisposition", "type": "str"},
    }

    def __init__(
        self,
        *,
        file_content_type: Optional[str] = None,
        file_content_encoding: Optional[str] = None,
        file_content_language: Optional[str] = None,
        file_cache_control: Optional[str] = None,
        file_content_md5: Optional[bytes] = None,
        file_content_disposition: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword file_content_type: Sets the MIME content type of the file. The default type is
         'application/octet-stream'.
        :paramtype file_content_type: str
        :keyword file_content_encoding: Specifies which content encodings have been applied to the
         file.
        :paramtype file_content_encoding: str
        :keyword file_content_language: Specifies the natural languages used by this resource.
        :paramtype file_content_language: str
        :keyword file_cache_control: Sets the file's cache control. The File service stores this value
         but does not use or modify it.
        :paramtype file_cache_control: str
        :keyword file_content_md5: Sets the file's MD5 hash.
        :paramtype file_content_md5: bytes
        :keyword file_content_disposition: Sets the file's Content-Disposition header.
        :paramtype file_content_disposition: str
        """
        super().__init__(**kwargs)
        self.file_content_type = file_content_type
        self.file_content_encoding = file_content_encoding
        self.file_content_language = file_content_language
        self.file_cache_control = file_cache_control
        self.file_content_md5 = file_content_md5
        self.file_content_disposition = file_content_disposition


class FileItem(_serialization.Model):
    """A listed file item.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required.
    :vartype name: ~azure.storage.fileshare.models.StringEncoded
    :ivar file_id:
    :vartype file_id: str
    :ivar properties: File properties. Required.
    :vartype properties: ~azure.storage.fileshare.models.FileProperty
    :ivar attributes:
    :vartype attributes: str
    :ivar permission_key:
    :vartype permission_key: str
    """

    _validation = {
        "name": {"required": True},
        "properties": {"required": True},
    }

    _attribute_map = {
        "name": {"key": "Name", "type": "StringEncoded"},
        "file_id": {"key": "FileId", "type": "str"},
        "properties": {"key": "Properties", "type": "FileProperty"},
        "attributes": {"key": "Attributes", "type": "str"},
        "permission_key": {"key": "PermissionKey", "type": "str"},
    }
    _xml_map = {"name": "File"}

    def __init__(
        self,
        *,
        name: "_models.StringEncoded",
        properties: "_models.FileProperty",
        file_id: Optional[str] = None,
        attributes: Optional[str] = None,
        permission_key: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: Required.
        :paramtype name: ~azure.storage.fileshare.models.StringEncoded
        :keyword file_id:
        :paramtype file_id: str
        :keyword properties: File properties. Required.
        :paramtype properties: ~azure.storage.fileshare.models.FileProperty
        :keyword attributes:
        :paramtype attributes: str
        :keyword permission_key:
        :paramtype permission_key: str
        """
        super().__init__(**kwargs)
        self.name = name
        self.file_id = file_id
        self.properties = properties
        self.attributes = attributes
        self.permission_key = permission_key


class FileProperty(_serialization.Model):
    """File properties.

    All required parameters must be populated in order to send to Azure.

    :ivar content_length: Content length of the file. This value may not be up-to-date since an SMB
     client may have modified the file locally. The value of Content-Length may not reflect that
     fact until the handle is closed or the op-lock is broken. To retrieve current property values,
     call Get File Properties. Required.
    :vartype content_length: int
    :ivar creation_time:
    :vartype creation_time: ~datetime.datetime
    :ivar last_access_time:
    :vartype last_access_time: ~datetime.datetime
    :ivar last_write_time:
    :vartype last_write_time: ~datetime.datetime
    :ivar change_time:
    :vartype change_time: ~datetime.datetime
    :ivar last_modified:
    :vartype last_modified: ~datetime.datetime
    :ivar etag:
    :vartype etag: str
    """

    _validation = {
        "content_length": {"required": True},
    }

    _attribute_map = {
        "content_length": {"key": "Content-Length", "type": "int"},
        "creation_time": {"key": "CreationTime", "type": "iso-8601"},
        "last_access_time": {"key": "LastAccessTime", "type": "iso-8601"},
        "last_write_time": {"key": "LastWriteTime", "type": "iso-8601"},
        "change_time": {"key": "ChangeTime", "type": "iso-8601"},
        "last_modified": {"key": "Last-Modified", "type": "rfc-1123"},
        "etag": {"key": "Etag", "type": "str"},
    }

    def __init__(
        self,
        *,
        content_length: int,
        creation_time: Optional[datetime.datetime] = None,
        last_access_time: Optional[datetime.datetime] = None,
        last_write_time: Optional[datetime.datetime] = None,
        change_time: Optional[datetime.datetime] = None,
        last_modified: Optional[datetime.datetime] = None,
        etag: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword content_length: Content length of the file. This value may not be up-to-date since an
         SMB client may have modified the file locally. The value of Content-Length may not reflect that
         fact until the handle is closed or the op-lock is broken. To retrieve current property values,
         call Get File Properties. Required.
        :paramtype content_length: int
        :keyword creation_time:
        :paramtype creation_time: ~datetime.datetime
        :keyword last_access_time:
        :paramtype last_access_time: ~datetime.datetime
        :keyword last_write_time:
        :paramtype last_write_time: ~datetime.datetime
        :keyword change_time:
        :paramtype change_time: ~datetime.datetime
        :keyword last_modified:
        :paramtype last_modified: ~datetime.datetime
        :keyword etag:
        :paramtype etag: str
        """
        super().__init__(**kwargs)
        self.content_length = content_length
        self.creation_time = creation_time
        self.last_access_time = last_access_time
        self.last_write_time = last_write_time
        self.change_time = change_time
        self.last_modified = last_modified
        self.etag = etag


class FileRange(_serialization.Model):
    """An Azure Storage file range.

    All required parameters must be populated in order to send to Azure.

    :ivar start: Start of the range. Required.
    :vartype start: int
    :ivar end: End of the range. Required.
    :vartype end: int
    """

    _validation = {
        "start": {"required": True},
        "end": {"required": True},
    }

    _attribute_map = {
        "start": {"key": "Start", "type": "int"},
        "end": {"key": "End", "type": "int"},
    }
    _xml_map = {"name": "Range"}

    def __init__(self, *, start: int, end: int, **kwargs: Any) -> None:
        """
        :keyword start: Start of the range. Required.
        :paramtype start: int
        :keyword end: End of the range. Required.
        :paramtype end: int
        """
        super().__init__(**kwargs)
        self.start = start
        self.end = end


class FilesAndDirectoriesListSegment(_serialization.Model):
    """Abstract for entries that can be listed from Directory.

    All required parameters must be populated in order to send to Azure.

    :ivar directory_items: Required.
    :vartype directory_items: list[~azure.storage.fileshare.models.DirectoryItem]
    :ivar file_items: Required.
    :vartype file_items: list[~azure.storage.fileshare.models.FileItem]
    """

    _validation = {
        "directory_items": {"required": True},
        "file_items": {"required": True},
    }

    _attribute_map = {
        "directory_items": {"key": "DirectoryItems", "type": "[DirectoryItem]", "xml": {"itemsName": "Directory"}},
        "file_items": {"key": "FileItems", "type": "[FileItem]", "xml": {"itemsName": "File"}},
    }
    _xml_map = {"name": "Entries"}

    def __init__(
        self, *, directory_items: List["_models.DirectoryItem"], file_items: List["_models.FileItem"], **kwargs: Any
    ) -> None:
        """
        :keyword directory_items: Required.
        :paramtype directory_items: list[~azure.storage.fileshare.models.DirectoryItem]
        :keyword file_items: Required.
        :paramtype file_items: list[~azure.storage.fileshare.models.FileItem]
        """
        super().__init__(**kwargs)
        self.directory_items = directory_items
        self.file_items = file_items


class HandleItem(_serialization.Model):
    """A listed Azure Storage handle item.

    All required parameters must be populated in order to send to Azure.

    :ivar handle_id: XSMB service handle ID. Required.
    :vartype handle_id: str
    :ivar path: Required.
    :vartype path: ~azure.storage.fileshare.models.StringEncoded
    :ivar file_id: FileId uniquely identifies the file or directory. Required.
    :vartype file_id: str
    :ivar parent_id: ParentId uniquely identifies the parent directory of the object.
    :vartype parent_id: str
    :ivar session_id: SMB session ID in context of which the file handle was opened. Required.
    :vartype session_id: str
    :ivar client_ip: Client IP that opened the handle. Required.
    :vartype client_ip: str
    :ivar open_time: Time when the session that previously opened the handle has last been
     reconnected. (UTC). Required.
    :vartype open_time: ~datetime.datetime
    :ivar last_reconnect_time: Time handle was last connected to (UTC).
    :vartype last_reconnect_time: ~datetime.datetime
    :ivar access_right_list:
    :vartype access_right_list: list[str or ~azure.storage.fileshare.models.AccessRight]
    """

    _validation = {
        "handle_id": {"required": True},
        "path": {"required": True},
        "file_id": {"required": True},
        "session_id": {"required": True},
        "client_ip": {"required": True},
        "open_time": {"required": True},
    }

    _attribute_map = {
        "handle_id": {"key": "HandleId", "type": "str"},
        "path": {"key": "Path", "type": "StringEncoded"},
        "file_id": {"key": "FileId", "type": "str"},
        "parent_id": {"key": "ParentId", "type": "str"},
        "session_id": {"key": "SessionId", "type": "str"},
        "client_ip": {"key": "ClientIp", "type": "str"},
        "open_time": {"key": "OpenTime", "type": "rfc-1123"},
        "last_reconnect_time": {"key": "LastReconnectTime", "type": "rfc-1123"},
        "access_right_list": {"key": "AccessRightList", "type": "[str]", "xml": {"wrapped": True}},
    }
    _xml_map = {"name": "Handle"}

    def __init__(
        self,
        *,
        handle_id: str,
        path: "_models.StringEncoded",
        file_id: str,
        session_id: str,
        client_ip: str,
        open_time: datetime.datetime,
        parent_id: Optional[str] = None,
        last_reconnect_time: Optional[datetime.datetime] = None,
        access_right_list: Optional[List[Union[str, "_models.AccessRight"]]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword handle_id: XSMB service handle ID. Required.
        :paramtype handle_id: str
        :keyword path: Required.
        :paramtype path: ~azure.storage.fileshare.models.StringEncoded
        :keyword file_id: FileId uniquely identifies the file or directory. Required.
        :paramtype file_id: str
        :keyword parent_id: ParentId uniquely identifies the parent directory of the object.
        :paramtype parent_id: str
        :keyword session_id: SMB session ID in context of which the file handle was opened. Required.
        :paramtype session_id: str
        :keyword client_ip: Client IP that opened the handle. Required.
        :paramtype client_ip: str
        :keyword open_time: Time when the session that previously opened the handle has last been
         reconnected. (UTC). Required.
        :paramtype open_time: ~datetime.datetime
        :keyword last_reconnect_time: Time handle was last connected to (UTC).
        :paramtype last_reconnect_time: ~datetime.datetime
        :keyword access_right_list:
        :paramtype access_right_list: list[str or ~azure.storage.fileshare.models.AccessRight]
        """
        super().__init__(**kwargs)
        self.handle_id = handle_id
        self.path = path
        self.file_id = file_id
        self.parent_id = parent_id
        self.session_id = session_id
        self.client_ip = client_ip
        self.open_time = open_time
        self.last_reconnect_time = last_reconnect_time
        self.access_right_list = access_right_list


class LeaseAccessConditions(_serialization.Model):
    """Parameter group.

    :ivar lease_id: If specified, the operation only succeeds if the resource's lease is active and
     matches this ID.
    :vartype lease_id: str
    """

    _attribute_map = {
        "lease_id": {"key": "leaseId", "type": "str"},
    }

    def __init__(self, *, lease_id: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword lease_id: If specified, the operation only succeeds if the resource's lease is active
         and matches this ID.
        :paramtype lease_id: str
        """
        super().__init__(**kwargs)
        self.lease_id = lease_id


class ListFilesAndDirectoriesSegmentResponse(_serialization.Model):  # pylint: disable=too-many-instance-attributes
    """An enumeration of directories and files.

    All required parameters must be populated in order to send to Azure.

    :ivar service_endpoint: Required.
    :vartype service_endpoint: str
    :ivar share_name: Required.
    :vartype share_name: str
    :ivar share_snapshot:
    :vartype share_snapshot: str
    :ivar encoded:
    :vartype encoded: bool
    :ivar directory_path: Required.
    :vartype directory_path: str
    :ivar prefix: Required.
    :vartype prefix: ~azure.storage.fileshare.models.StringEncoded
    :ivar marker:
    :vartype marker: str
    :ivar max_results:
    :vartype max_results: int
    :ivar segment: Abstract for entries that can be listed from Directory. Required.
    :vartype segment: ~azure.storage.fileshare.models.FilesAndDirectoriesListSegment
    :ivar next_marker: Required.
    :vartype next_marker: str
    :ivar directory_id:
    :vartype directory_id: str
    """

    _validation = {
        "service_endpoint": {"required": True},
        "share_name": {"required": True},
        "directory_path": {"required": True},
        "prefix": {"required": True},
        "segment": {"required": True},
        "next_marker": {"required": True},
    }

    _attribute_map = {
        "service_endpoint": {"key": "ServiceEndpoint", "type": "str", "xml": {"attr": True}},
        "share_name": {"key": "ShareName", "type": "str", "xml": {"attr": True}},
        "share_snapshot": {"key": "ShareSnapshot", "type": "str", "xml": {"attr": True}},
        "encoded": {"key": "Encoded", "type": "bool", "xml": {"attr": True}},
        "directory_path": {"key": "DirectoryPath", "type": "str", "xml": {"attr": True}},
        "prefix": {"key": "Prefix", "type": "StringEncoded"},
        "marker": {"key": "Marker", "type": "str"},
        "max_results": {"key": "MaxResults", "type": "int"},
        "segment": {"key": "Segment", "type": "FilesAndDirectoriesListSegment"},
        "next_marker": {"key": "NextMarker", "type": "str"},
        "directory_id": {"key": "DirectoryId", "type": "str"},
    }
    _xml_map = {"name": "EnumerationResults"}

    def __init__(
        self,
        *,
        service_endpoint: str,
        share_name: str,
        directory_path: str,
        prefix: "_models.StringEncoded",
        segment: "_models.FilesAndDirectoriesListSegment",
        next_marker: str,
        share_snapshot: Optional[str] = None,
        encoded: Optional[bool] = None,
        marker: Optional[str] = None,
        max_results: Optional[int] = None,
        directory_id: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword service_endpoint: Required.
        :paramtype service_endpoint: str
        :keyword share_name: Required.
        :paramtype share_name: str
        :keyword share_snapshot:
        :paramtype share_snapshot: str
        :keyword encoded:
        :paramtype encoded: bool
        :keyword directory_path: Required.
        :paramtype directory_path: str
        :keyword prefix: Required.
        :paramtype prefix: ~azure.storage.fileshare.models.StringEncoded
        :keyword marker:
        :paramtype marker: str
        :keyword max_results:
        :paramtype max_results: int
        :keyword segment: Abstract for entries that can be listed from Directory. Required.
        :paramtype segment: ~azure.storage.fileshare.models.FilesAndDirectoriesListSegment
        :keyword next_marker: Required.
        :paramtype next_marker: str
        :keyword directory_id:
        :paramtype directory_id: str
        """
        super().__init__(**kwargs)
        self.service_endpoint = service_endpoint
        self.share_name = share_name
        self.share_snapshot = share_snapshot
        self.encoded = encoded
        self.directory_path = directory_path
        self.prefix = prefix
        self.marker = marker
        self.max_results = max_results
        self.segment = segment
        self.next_marker = next_marker
        self.directory_id = directory_id


class ListHandlesResponse(_serialization.Model):
    """An enumeration of handles.

    All required parameters must be populated in order to send to Azure.

    :ivar handle_list:
    :vartype handle_list: list[~azure.storage.fileshare.models.HandleItem]
    :ivar next_marker: Required.
    :vartype next_marker: str
    """

    _validation = {
        "next_marker": {"required": True},
    }

    _attribute_map = {
        "handle_list": {
            "key": "HandleList",
            "type": "[HandleItem]",
            "xml": {"name": "Entries", "wrapped": True, "itemsName": "Handle"},
        },
        "next_marker": {"key": "NextMarker", "type": "str"},
    }
    _xml_map = {"name": "EnumerationResults"}

    def __init__(
        self, *, next_marker: str, handle_list: Optional[List["_models.HandleItem"]] = None, **kwargs: Any
    ) -> None:
        """
        :keyword handle_list:
        :paramtype handle_list: list[~azure.storage.fileshare.models.HandleItem]
        :keyword next_marker: Required.
        :paramtype next_marker: str
        """
        super().__init__(**kwargs)
        self.handle_list = handle_list
        self.next_marker = next_marker


class ListSharesResponse(_serialization.Model):
    """An enumeration of shares.

    All required parameters must be populated in order to send to Azure.

    :ivar service_endpoint: Required.
    :vartype service_endpoint: str
    :ivar prefix:
    :vartype prefix: str
    :ivar marker:
    :vartype marker: str
    :ivar max_results:
    :vartype max_results: int
    :ivar share_items:
    :vartype share_items: list[~azure.storage.fileshare.models.ShareItemInternal]
    :ivar next_marker: Required.
    :vartype next_marker: str
    """

    _validation = {
        "service_endpoint": {"required": True},
        "next_marker": {"required": True},
    }

    _attribute_map = {
        "service_endpoint": {"key": "ServiceEndpoint", "type": "str", "xml": {"attr": True}},
        "prefix": {"key": "Prefix", "type": "str"},
        "marker": {"key": "Marker", "type": "str"},
        "max_results": {"key": "MaxResults", "type": "int"},
        "share_items": {
            "key": "ShareItems",
            "type": "[ShareItemInternal]",
            "xml": {"name": "Shares", "wrapped": True, "itemsName": "Share"},
        },
        "next_marker": {"key": "NextMarker", "type": "str"},
    }
    _xml_map = {"name": "EnumerationResults"}

    def __init__(
        self,
        *,
        service_endpoint: str,
        next_marker: str,
        prefix: Optional[str] = None,
        marker: Optional[str] = None,
        max_results: Optional[int] = None,
        share_items: Optional[List["_models.ShareItemInternal"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword service_endpoint: Required.
        :paramtype service_endpoint: str
        :keyword prefix:
        :paramtype prefix: str
        :keyword marker:
        :paramtype marker: str
        :keyword max_results:
        :paramtype max_results: int
        :keyword share_items:
        :paramtype share_items: list[~azure.storage.fileshare.models.ShareItemInternal]
        :keyword next_marker: Required.
        :paramtype next_marker: str
        """
        super().__init__(**kwargs)
        self.service_endpoint = service_endpoint
        self.prefix = prefix
        self.marker = marker
        self.max_results = max_results
        self.share_items = share_items
        self.next_marker = next_marker


class Metrics(_serialization.Model):
    """Storage Analytics metrics for file service.

    All required parameters must be populated in order to send to Azure.

    :ivar version: The version of Storage Analytics to configure. Required.
    :vartype version: str
    :ivar enabled: Indicates whether metrics are enabled for the File service. Required.
    :vartype enabled: bool
    :ivar include_apis: Indicates whether metrics should generate summary statistics for called API
     operations.
    :vartype include_apis: bool
    :ivar retention_policy: The retention policy.
    :vartype retention_policy: ~azure.storage.fileshare.models.RetentionPolicy
    """

    _validation = {
        "version": {"required": True},
        "enabled": {"required": True},
    }

    _attribute_map = {
        "version": {"key": "Version", "type": "str"},
        "enabled": {"key": "Enabled", "type": "bool"},
        "include_apis": {"key": "IncludeAPIs", "type": "bool"},
        "retention_policy": {"key": "RetentionPolicy", "type": "RetentionPolicy"},
    }

    def __init__(
        self,
        *,
        version: str,
        enabled: bool,
        include_apis: Optional[bool] = None,
        retention_policy: Optional["_models.RetentionPolicy"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword version: The version of Storage Analytics to configure. Required.
        :paramtype version: str
        :keyword enabled: Indicates whether metrics are enabled for the File service. Required.
        :paramtype enabled: bool
        :keyword include_apis: Indicates whether metrics should generate summary statistics for called
         API operations.
        :paramtype include_apis: bool
        :keyword retention_policy: The retention policy.
        :paramtype retention_policy: ~azure.storage.fileshare.models.RetentionPolicy
        """
        super().__init__(**kwargs)
        self.version = version
        self.enabled = enabled
        self.include_apis = include_apis
        self.retention_policy = retention_policy


class RetentionPolicy(_serialization.Model):
    """The retention policy.

    All required parameters must be populated in order to send to Azure.

    :ivar enabled: Indicates whether a retention policy is enabled for the File service. If false,
     metrics data is retained, and the user is responsible for deleting it. Required.
    :vartype enabled: bool
    :ivar days: Indicates the number of days that metrics data should be retained. All data older
     than this value will be deleted. Metrics data is deleted on a best-effort basis after the
     retention period expires.
    :vartype days: int
    """

    _validation = {
        "enabled": {"required": True},
        "days": {"maximum": 365, "minimum": 1},
    }

    _attribute_map = {
        "enabled": {"key": "Enabled", "type": "bool"},
        "days": {"key": "Days", "type": "int"},
    }

    def __init__(self, *, enabled: bool, days: Optional[int] = None, **kwargs: Any) -> None:
        """
        :keyword enabled: Indicates whether a retention policy is enabled for the File service. If
         false, metrics data is retained, and the user is responsible for deleting it. Required.
        :paramtype enabled: bool
        :keyword days: Indicates the number of days that metrics data should be retained. All data
         older than this value will be deleted. Metrics data is deleted on a best-effort basis after the
         retention period expires.
        :paramtype days: int
        """
        super().__init__(**kwargs)
        self.enabled = enabled
        self.days = days


class ShareFileRangeList(_serialization.Model):
    """The list of file ranges.

    :ivar ranges:
    :vartype ranges: list[~azure.storage.fileshare.models.FileRange]
    :ivar clear_ranges:
    :vartype clear_ranges: list[~azure.storage.fileshare.models.ClearRange]
    """

    _attribute_map = {
        "ranges": {"key": "Ranges", "type": "[FileRange]", "xml": {"itemsName": "Range"}},
        "clear_ranges": {"key": "ClearRanges", "type": "[ClearRange]", "xml": {"itemsName": "ClearRange"}},
    }

    def __init__(
        self,
        *,
        ranges: Optional[List["_models.FileRange"]] = None,
        clear_ranges: Optional[List["_models.ClearRange"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword ranges:
        :paramtype ranges: list[~azure.storage.fileshare.models.FileRange]
        :keyword clear_ranges:
        :paramtype clear_ranges: list[~azure.storage.fileshare.models.ClearRange]
        """
        super().__init__(**kwargs)
        self.ranges = ranges
        self.clear_ranges = clear_ranges


class ShareItemInternal(_serialization.Model):
    """A listed Azure Storage share item.

    All required parameters must be populated in order to send to Azure.

    :ivar name: Required.
    :vartype name: str
    :ivar snapshot:
    :vartype snapshot: str
    :ivar deleted:
    :vartype deleted: bool
    :ivar version:
    :vartype version: str
    :ivar properties: Properties of a share. Required.
    :vartype properties: ~azure.storage.fileshare.models.SharePropertiesInternal
    :ivar metadata: Dictionary of :code:`<string>`.
    :vartype metadata: dict[str, str]
    """

    _validation = {
        "name": {"required": True},
        "properties": {"required": True},
    }

    _attribute_map = {
        "name": {"key": "Name", "type": "str"},
        "snapshot": {"key": "Snapshot", "type": "str"},
        "deleted": {"key": "Deleted", "type": "bool"},
        "version": {"key": "Version", "type": "str"},
        "properties": {"key": "Properties", "type": "SharePropertiesInternal"},
        "metadata": {"key": "Metadata", "type": "{str}"},
    }
    _xml_map = {"name": "Share"}

    def __init__(
        self,
        *,
        name: str,
        properties: "_models.SharePropertiesInternal",
        snapshot: Optional[str] = None,
        deleted: Optional[bool] = None,
        version: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: Required.
        :paramtype name: str
        :keyword snapshot:
        :paramtype snapshot: str
        :keyword deleted:
        :paramtype deleted: bool
        :keyword version:
        :paramtype version: str
        :keyword properties: Properties of a share. Required.
        :paramtype properties: ~azure.storage.fileshare.models.SharePropertiesInternal
        :keyword metadata: Dictionary of :code:`<string>`.
        :paramtype metadata: dict[str, str]
        """
        super().__init__(**kwargs)
        self.name = name
        self.snapshot = snapshot
        self.deleted = deleted
        self.version = version
        self.properties = properties
        self.metadata = metadata


class SharePermission(_serialization.Model):
    """A permission (a security descriptor) at the share level.

    All required parameters must be populated in order to send to Azure.

    :ivar permission: The permission in the Security Descriptor Definition Language (SDDL).
     Required.
    :vartype permission: str
    """

    _validation = {
        "permission": {"required": True},
    }

    _attribute_map = {
        "permission": {"key": "permission", "type": "str"},
    }

    def __init__(self, *, permission: str, **kwargs: Any) -> None:
        """
        :keyword permission: The permission in the Security Descriptor Definition Language (SDDL).
         Required.
        :paramtype permission: str
        """
        super().__init__(**kwargs)
        self.permission = permission


class SharePropertiesInternal(_serialization.Model):  # pylint: disable=too-many-instance-attributes
    """Properties of a share.

    All required parameters must be populated in order to send to Azure.

    :ivar last_modified: Required.
    :vartype last_modified: ~datetime.datetime
    :ivar etag: Required.
    :vartype etag: str
    :ivar quota: Required.
    :vartype quota: int
    :ivar provisioned_iops:
    :vartype provisioned_iops: int
    :ivar provisioned_ingress_m_bps:
    :vartype provisioned_ingress_m_bps: int
    :ivar provisioned_egress_m_bps:
    :vartype provisioned_egress_m_bps: int
    :ivar provisioned_bandwidth_mi_bps:
    :vartype provisioned_bandwidth_mi_bps: int
    :ivar next_allowed_quota_downgrade_time:
    :vartype next_allowed_quota_downgrade_time: ~datetime.datetime
    :ivar deleted_time:
    :vartype deleted_time: ~datetime.datetime
    :ivar remaining_retention_days:
    :vartype remaining_retention_days: int
    :ivar access_tier:
    :vartype access_tier: str
    :ivar access_tier_change_time:
    :vartype access_tier_change_time: ~datetime.datetime
    :ivar access_tier_transition_state:
    :vartype access_tier_transition_state: str
    :ivar lease_status: The current lease status of the share. Known values are: "locked" and
     "unlocked".
    :vartype lease_status: str or ~azure.storage.fileshare.models.LeaseStatusType
    :ivar lease_state: Lease state of the share. Known values are: "available", "leased",
     "expired", "breaking", and "broken".
    :vartype lease_state: str or ~azure.storage.fileshare.models.LeaseStateType
    :ivar lease_duration: When a share is leased, specifies whether the lease is of infinite or
     fixed duration. Known values are: "infinite" and "fixed".
    :vartype lease_duration: str or ~azure.storage.fileshare.models.LeaseDurationType
    :ivar enabled_protocols:
    :vartype enabled_protocols: str
    :ivar root_squash: Known values are: "NoRootSquash", "RootSquash", and "AllSquash".
    :vartype root_squash: str or ~azure.storage.fileshare.models.ShareRootSquash
    """

    _validation = {
        "last_modified": {"required": True},
        "etag": {"required": True},
        "quota": {"required": True},
    }

    _attribute_map = {
        "last_modified": {"key": "Last-Modified", "type": "rfc-1123"},
        "etag": {"key": "Etag", "type": "str"},
        "quota": {"key": "Quota", "type": "int"},
        "provisioned_iops": {"key": "ProvisionedIops", "type": "int"},
        "provisioned_ingress_m_bps": {"key": "ProvisionedIngressMBps", "type": "int"},
        "provisioned_egress_m_bps": {"key": "ProvisionedEgressMBps", "type": "int"},
        "provisioned_bandwidth_mi_bps": {"key": "ProvisionedBandwidthMiBps", "type": "int"},
        "next_allowed_quota_downgrade_time": {"key": "NextAllowedQuotaDowngradeTime", "type": "rfc-1123"},
        "deleted_time": {"key": "DeletedTime", "type": "rfc-1123"},
        "remaining_retention_days": {"key": "RemainingRetentionDays", "type": "int"},
        "access_tier": {"key": "AccessTier", "type": "str"},
        "access_tier_change_time": {"key": "AccessTierChangeTime", "type": "rfc-1123"},
        "access_tier_transition_state": {"key": "AccessTierTransitionState", "type": "str"},
        "lease_status": {"key": "LeaseStatus", "type": "str"},
        "lease_state": {"key": "LeaseState", "type": "str"},
        "lease_duration": {"key": "LeaseDuration", "type": "str"},
        "enabled_protocols": {"key": "EnabledProtocols", "type": "str"},
        "root_squash": {"key": "RootSquash", "type": "str"},
    }

    def __init__(
        self,
        *,
        last_modified: datetime.datetime,
        etag: str,
        quota: int,
        provisioned_iops: Optional[int] = None,
        provisioned_ingress_m_bps: Optional[int] = None,
        provisioned_egress_m_bps: Optional[int] = None,
        provisioned_bandwidth_mi_bps: Optional[int] = None,
        next_allowed_quota_downgrade_time: Optional[datetime.datetime] = None,
        deleted_time: Optional[datetime.datetime] = None,
        remaining_retention_days: Optional[int] = None,
        access_tier: Optional[str] = None,
        access_tier_change_time: Optional[datetime.datetime] = None,
        access_tier_transition_state: Optional[str] = None,
        lease_status: Optional[Union[str, "_models.LeaseStatusType"]] = None,
        lease_state: Optional[Union[str, "_models.LeaseStateType"]] = None,
        lease_duration: Optional[Union[str, "_models.LeaseDurationType"]] = None,
        enabled_protocols: Optional[str] = None,
        root_squash: Optional[Union[str, "_models.ShareRootSquash"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword last_modified: Required.
        :paramtype last_modified: ~datetime.datetime
        :keyword etag: Required.
        :paramtype etag: str
        :keyword quota: Required.
        :paramtype quota: int
        :keyword provisioned_iops:
        :paramtype provisioned_iops: int
        :keyword provisioned_ingress_m_bps:
        :paramtype provisioned_ingress_m_bps: int
        :keyword provisioned_egress_m_bps:
        :paramtype provisioned_egress_m_bps: int
        :keyword provisioned_bandwidth_mi_bps:
        :paramtype provisioned_bandwidth_mi_bps: int
        :keyword next_allowed_quota_downgrade_time:
        :paramtype next_allowed_quota_downgrade_time: ~datetime.datetime
        :keyword deleted_time:
        :paramtype deleted_time: ~datetime.datetime
        :keyword remaining_retention_days:
        :paramtype remaining_retention_days: int
        :keyword access_tier:
        :paramtype access_tier: str
        :keyword access_tier_change_time:
        :paramtype access_tier_change_time: ~datetime.datetime
        :keyword access_tier_transition_state:
        :paramtype access_tier_transition_state: str
        :keyword lease_status: The current lease status of the share. Known values are: "locked" and
         "unlocked".
        :paramtype lease_status: str or ~azure.storage.fileshare.models.LeaseStatusType
        :keyword lease_state: Lease state of the share. Known values are: "available", "leased",
         "expired", "breaking", and "broken".
        :paramtype lease_state: str or ~azure.storage.fileshare.models.LeaseStateType
        :keyword lease_duration: When a share is leased, specifies whether the lease is of infinite or
         fixed duration. Known values are: "infinite" and "fixed".
        :paramtype lease_duration: str or ~azure.storage.fileshare.models.LeaseDurationType
        :keyword enabled_protocols:
        :paramtype enabled_protocols: str
        :keyword root_squash: Known values are: "NoRootSquash", "RootSquash", and "AllSquash".
        :paramtype root_squash: str or ~azure.storage.fileshare.models.ShareRootSquash
        """
        super().__init__(**kwargs)
        self.last_modified = last_modified
        self.etag = etag
        self.quota = quota
        self.provisioned_iops = provisioned_iops
        self.provisioned_ingress_m_bps = provisioned_ingress_m_bps
        self.provisioned_egress_m_bps = provisioned_egress_m_bps
        self.provisioned_bandwidth_mi_bps = provisioned_bandwidth_mi_bps
        self.next_allowed_quota_downgrade_time = next_allowed_quota_downgrade_time
        self.deleted_time = deleted_time
        self.remaining_retention_days = remaining_retention_days
        self.access_tier = access_tier
        self.access_tier_change_time = access_tier_change_time
        self.access_tier_transition_state = access_tier_transition_state
        self.lease_status = lease_status
        self.lease_state = lease_state
        self.lease_duration = lease_duration
        self.enabled_protocols = enabled_protocols
        self.root_squash = root_squash


class ShareProtocolSettings(_serialization.Model):
    """Protocol settings.

    :ivar smb: Settings for SMB protocol.
    :vartype smb: ~azure.storage.fileshare.models.ShareSmbSettings
    """

    _attribute_map = {
        "smb": {"key": "Smb", "type": "ShareSmbSettings"},
    }
    _xml_map = {"name": "ProtocolSettings"}

    def __init__(self, *, smb: Optional["_models.ShareSmbSettings"] = None, **kwargs: Any) -> None:
        """
        :keyword smb: Settings for SMB protocol.
        :paramtype smb: ~azure.storage.fileshare.models.ShareSmbSettings
        """
        super().__init__(**kwargs)
        self.smb = smb


class ShareSmbSettings(_serialization.Model):
    """Settings for SMB protocol.

    :ivar multichannel: Settings for SMB Multichannel.
    :vartype multichannel: ~azure.storage.fileshare.models.SmbMultichannel
    """

    _attribute_map = {
        "multichannel": {"key": "Multichannel", "type": "SmbMultichannel"},
    }
    _xml_map = {"name": "SMB"}

    def __init__(self, *, multichannel: Optional["_models.SmbMultichannel"] = None, **kwargs: Any) -> None:
        """
        :keyword multichannel: Settings for SMB Multichannel.
        :paramtype multichannel: ~azure.storage.fileshare.models.SmbMultichannel
        """
        super().__init__(**kwargs)
        self.multichannel = multichannel


class ShareStats(_serialization.Model):
    """Stats for the share.

    All required parameters must be populated in order to send to Azure.

    :ivar share_usage_bytes: The approximate size of the data stored in bytes. Note that this value
     may not include all recently created or recently resized files. Required.
    :vartype share_usage_bytes: int
    """

    _validation = {
        "share_usage_bytes": {"required": True},
    }

    _attribute_map = {
        "share_usage_bytes": {"key": "ShareUsageBytes", "type": "int"},
    }

    def __init__(self, *, share_usage_bytes: int, **kwargs: Any) -> None:
        """
        :keyword share_usage_bytes: The approximate size of the data stored in bytes. Note that this
         value may not include all recently created or recently resized files. Required.
        :paramtype share_usage_bytes: int
        """
        super().__init__(**kwargs)
        self.share_usage_bytes = share_usage_bytes


class SignedIdentifier(_serialization.Model):
    """Signed identifier.

    All required parameters must be populated in order to send to Azure.

    :ivar id: A unique id. Required.
    :vartype id: str
    :ivar access_policy: The access policy.
    :vartype access_policy: ~azure.storage.fileshare.models.AccessPolicy
    """

    _validation = {
        "id": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "Id", "type": "str"},
        "access_policy": {"key": "AccessPolicy", "type": "AccessPolicy"},
    }

    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        access_policy: Optional["_models.AccessPolicy"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword id: A unique id. Required.
        :paramtype id: str
        :keyword access_policy: The access policy.
        :paramtype access_policy: ~azure.storage.fileshare.models.AccessPolicy
        """
        super().__init__(**kwargs)
        self.id = id
        self.access_policy = access_policy


class SmbMultichannel(_serialization.Model):
    """Settings for SMB multichannel.

    :ivar enabled: If SMB multichannel is enabled.
    :vartype enabled: bool
    """

    _attribute_map = {
        "enabled": {"key": "Enabled", "type": "bool"},
    }
    _xml_map = {"name": "Multichannel"}

    def __init__(self, *, enabled: Optional[bool] = None, **kwargs: Any) -> None:
        """
        :keyword enabled: If SMB multichannel is enabled.
        :paramtype enabled: bool
        """
        super().__init__(**kwargs)
        self.enabled = enabled


class SourceLeaseAccessConditions(_serialization.Model):
    """Parameter group.

    :ivar source_lease_id: Required if the source file has an active infinite lease.
    :vartype source_lease_id: str
    """

    _attribute_map = {
        "source_lease_id": {"key": "sourceLeaseId", "type": "str"},
    }

    def __init__(self, *, source_lease_id: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword source_lease_id: Required if the source file has an active infinite lease.
        :paramtype source_lease_id: str
        """
        super().__init__(**kwargs)
        self.source_lease_id = source_lease_id


class SourceModifiedAccessConditions(_serialization.Model):
    """Parameter group.

    :ivar source_if_match_crc64: Specify the crc64 value to operate only on range with a matching
     crc64 checksum.
    :vartype source_if_match_crc64: bytes
    :ivar source_if_none_match_crc64: Specify the crc64 value to operate only on range without a
     matching crc64 checksum.
    :vartype source_if_none_match_crc64: bytes
    """

    _attribute_map = {
        "source_if_match_crc64": {"key": "sourceIfMatchCrc64", "type": "bytearray"},
        "source_if_none_match_crc64": {"key": "sourceIfNoneMatchCrc64", "type": "bytearray"},
    }

    def __init__(
        self,
        *,
        source_if_match_crc64: Optional[bytes] = None,
        source_if_none_match_crc64: Optional[bytes] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword source_if_match_crc64: Specify the crc64 value to operate only on range with a
         matching crc64 checksum.
        :paramtype source_if_match_crc64: bytes
        :keyword source_if_none_match_crc64: Specify the crc64 value to operate only on range without a
         matching crc64 checksum.
        :paramtype source_if_none_match_crc64: bytes
        """
        super().__init__(**kwargs)
        self.source_if_match_crc64 = source_if_match_crc64
        self.source_if_none_match_crc64 = source_if_none_match_crc64


class StorageError(_serialization.Model):
    """StorageError.

    :ivar message:
    :vartype message: str
    """

    _attribute_map = {
        "message": {"key": "Message", "type": "str"},
    }

    def __init__(self, *, message: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword message:
        :paramtype message: str
        """
        super().__init__(**kwargs)
        self.message = message


class StorageServiceProperties(_serialization.Model):
    """Storage service properties.

    :ivar hour_metrics: A summary of request statistics grouped by API in hourly aggregates for
     files.
    :vartype hour_metrics: ~azure.storage.fileshare.models.Metrics
    :ivar minute_metrics: A summary of request statistics grouped by API in minute aggregates for
     files.
    :vartype minute_metrics: ~azure.storage.fileshare.models.Metrics
    :ivar cors: The set of CORS rules.
    :vartype cors: list[~azure.storage.fileshare.models.CorsRule]
    :ivar protocol: Protocol settings.
    :vartype protocol: ~azure.storage.fileshare.models.ShareProtocolSettings
    """

    _attribute_map = {
        "hour_metrics": {"key": "HourMetrics", "type": "Metrics"},
        "minute_metrics": {"key": "MinuteMetrics", "type": "Metrics"},
        "cors": {"key": "Cors", "type": "[CorsRule]", "xml": {"wrapped": True}},
        "protocol": {"key": "Protocol", "type": "ShareProtocolSettings"},
    }

    def __init__(
        self,
        *,
        hour_metrics: Optional["_models.Metrics"] = None,
        minute_metrics: Optional["_models.Metrics"] = None,
        cors: Optional[List["_models.CorsRule"]] = None,
        protocol: Optional["_models.ShareProtocolSettings"] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword hour_metrics: A summary of request statistics grouped by API in hourly aggregates for
         files.
        :paramtype hour_metrics: ~azure.storage.fileshare.models.Metrics
        :keyword minute_metrics: A summary of request statistics grouped by API in minute aggregates
         for files.
        :paramtype minute_metrics: ~azure.storage.fileshare.models.Metrics
        :keyword cors: The set of CORS rules.
        :paramtype cors: list[~azure.storage.fileshare.models.CorsRule]
        :keyword protocol: Protocol settings.
        :paramtype protocol: ~azure.storage.fileshare.models.ShareProtocolSettings
        """
        super().__init__(**kwargs)
        self.hour_metrics = hour_metrics
        self.minute_metrics = minute_metrics
        self.cors = cors
        self.protocol = protocol


class StringEncoded(_serialization.Model):
    """StringEncoded.

    :ivar encoded:
    :vartype encoded: bool
    :ivar content:
    :vartype content: str
    """

    _attribute_map = {
        "encoded": {"key": "Encoded", "type": "bool", "xml": {"name": "Encoded", "attr": True}},
        "content": {"key": "content", "type": "str", "xml": {"text": True}},
    }

    def __init__(self, *, encoded: Optional[bool] = None, content: Optional[str] = None, **kwargs: Any) -> None:
        """
        :keyword encoded:
        :paramtype encoded: bool
        :keyword content:
        :paramtype content: str
        """
        super().__init__(**kwargs)
        self.encoded = encoded
        self.content = content



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

from typing import Optional


class ProgressTracker:
    def __init__(self, total: int, step: int):
        self.total = total
        self.step = step
        self.current = 0

    async def assert_progress(self, current: int, total: Optional[int]):
        if self.current != self.total:
            self.current += self.step

        if total:
            assert self.total == total
        assert self.current == current

    def assert_complete(self):
        assert self.total == self.current



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.padding import (
    OAEP,
    MGF1,
)
from cryptography.hazmat.primitives.asymmetric.rsa import generate_private_key
from cryptography.hazmat.primitives.hashes import SHA1
from cryptography.hazmat.primitives.keywrap import (
    aes_key_wrap,
    aes_key_unwrap,
)


class KeyWrapper:
    def __init__(self, kid='local:key1'):
        # Must have constant key value for recorded tests, otherwise we could use a random generator.
        self.kek = b'\xbe\xa4\x11K\x9eJ\x07\xdafF\x83\xad+\xadvA C\xe8\xbc\x90\xa4\x11}G\xc3\x0f\xd4\xb4\x19m\x11'
        self.backend = default_backend()
        self.kid = kid

    def wrap_key(self, key, algorithm='A256KW'):
        if algorithm == 'A256KW':
            return aes_key_wrap(self.kek, key, self.backend)

        raise ValueError(_ERROR_UNKNOWN_KEY_WRAP_ALGORITHM)

    def unwrap_key(self, key, algorithm):
        if algorithm == 'A256KW':
            return aes_key_unwrap(self.kek, key, self.backend)

        raise ValueError(_ERROR_UNKNOWN_KEY_WRAP_ALGORITHM)

    def get_key_wrap_algorithm(self):
        return 'A256KW'

    def get_kid(self):
        return self.kid


class KeyResolver:
    def __init__(self):
        self.keys = {}

    def put_key(self, key):
        self.keys[key.get_kid()] = key

    def resolve_key(self, kid):
        return self.keys[kid]


class RSAKeyWrapper:
    def __init__(self, kid='local:key2'):
        self.private_key = generate_private_key(public_exponent=65537,
                                                key_size=2048,
                                                backend=default_backend())
        self.public_key = self.private_key.public_key()
        self.kid = kid

    def wrap_key(self, key, algorithm='RSA'):
        if algorithm == 'RSA':
            return self.public_key.encrypt(key,
                                           OAEP(
                                               mgf=MGF1(algorithm=SHA1()),  # nosec
                                               algorithm=SHA1(),    # nosec
                                               label=None)
                                           )

        raise ValueError(_ERROR_UNKNOWN_KEY_WRAP_ALGORITHM)

    def unwrap_key(self, key, algorithm):
        if algorithm == 'RSA':
            return self.private_key.decrypt(key,
                                            OAEP(
                                                mgf=MGF1(algorithm=SHA1()), # nosec
                                                algorithm=SHA1(),   # nosec
                                                label=None)
                                            )

        raise ValueError(_ERROR_UNKNOWN_KEY_WRAP_ALGORITHM)

    def get_key_wrap_algorithm(self):
        return 'RSA'

    def get_kid(self):
        return self.kid



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: file_samples_authentication.py

DESCRIPTION:
    These samples demonstrate authenticating a client via a connection string,
    shared access key, or by generating a sas token with which the returned signature
    can be used with the credential parameter of any ShareServiceClient,
    ShareClient, ShareDirectoryClient, or ShareFileClient.

USAGE:
    python file_samples_authentication.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_STORAGE_CONNECTION_STRING - the connection string to your storage account
    2) AZURE_STORAGE_ACCOUNT_URL - the queue service account URL
    3) AZURE_STORAGE_ACCOUNT_NAME - the name of the storage account
    4) AZURE_STORAGE_ACCESS_KEY - the storage account access key
"""

import os
from datetime import datetime, timedelta


class FileAuthSamples(object):

    connection_string = os.getenv("AZURE_STORAGE_CONNECTION_STRING")

    account_url = os.getenv("AZURE_STORAGE_ACCOUNT_URL")
    account_name = os.getenv("AZURE_STORAGE_ACCOUNT_NAME")
    access_key = os.getenv("AZURE_STORAGE_ACCESS_KEY")

    def authentication_connection_string(self):
        # Instantiate the ShareServiceClient from a connection string
        # [START create_share_service_client_from_conn_string]
        from azure.storage.fileshare import ShareServiceClient
        share_service_client = ShareServiceClient.from_connection_string(self.connection_string)
        # [END create_share_service_client_from_conn_string]

    def authentication_shared_access_key(self):
        # Instantiate a ShareServiceClient using a shared access key
        # [START create_share_service_client]
        from azure.storage.fileshare import ShareServiceClient
        share_service_client = ShareServiceClient(
            account_url=self.account_url,
            credential=self.access_key
        )
        # [END create_share_service_client]

    def authentication_shared_access_signature(self):
        # Instantiate a ShareServiceClient using a connection string
        # [START generate_sas_token]
        from azure.storage.fileshare import ShareServiceClient
        share_service_client = ShareServiceClient.from_connection_string(self.connection_string)

        # Create a SAS token to use to authenticate a new client
        from azure.storage.fileshare import generate_account_sas, ResourceTypes, AccountSasPermissions

        sas_token = generate_account_sas(
            self.account_name,
            self.access_key,
            resource_types=ResourceTypes(service=True),
            permission=AccountSasPermissions(read=True),
            expiry=datetime.utcnow() + timedelta(hours=1)
        )
        # [END generate_sas_token]

    def authentication_default_azure_credential(self):
        # [START file_share_oauth]
        # Get a credential for authentication
        # DefaultAzureCredential attempts a chained set of authentication methods.
        # See documentation here: https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/identity/azure-identity
        from azure.identity import DefaultAzureCredential
        default_credential = DefaultAzureCredential()

        # Instantiate a ShareServiceClient using a token credential and token_intent
        from azure.storage.fileshare import ShareServiceClient
        share_service_client = ShareServiceClient(
            account_url=self.account_url,
            credential=default_credential,
            # When using a token credential, you MUST also specify a token_intent
            token_intent='backup'
        )

        # Only Directory and File operations, and a certain few Share operations, are currently supported for OAuth.
        # Create a ShareFileClient from the ShareServiceClient.
        share_client = share_service_client.get_share_client('myshare')
        share_file_client = share_client.get_file_client('mydirectory/myfile')

        properties = share_file_client.get_file_properties()
        # [END file_share_oauth]


if __name__ == '__main__':
    sample = FileAuthSamples()
    sample.authentication_connection_string()
    sample.authentication_shared_access_key()
    sample.authentication_shared_access_signature()
    sample.authentication_default_azure_credential()




# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: file_samples_share.py

DESCRIPTION:
    These samples demonstrate share operations like creating a share snapshot,
    setting share quota and metadata, listing directories and files in the
    file share, and getting directory and file clients from a share client.

USAGE:
    python file_samples_share.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_STORAGE_CONNECTION_STRING - the connection string to your storage account
"""

import os
from azure.storage.fileshare import ShareAccessTier

SOURCE_FILE = './SampleSource.txt'
DEST_FILE = './SampleDestination.txt'


class ShareSamples(object):

    connection_string = os.getenv('AZURE_STORAGE_CONNECTION_STRING')

    def create_share_snapshot(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples1")

        # [START create_share]
        # Create share with Access Tier set to Hot
        share.create_share(access_tier=ShareAccessTier("Hot"))
        # [END create_share]
        try:
            # [START create_share_snapshot]
            share.create_snapshot()
            # [END create_share_snapshot]
        finally:
            # [START delete_share]
            share.delete_share(delete_snapshots=True)
            # [END delete_share]

    def set_share_quota_and_metadata(self):
        # [START create_share_client_from_conn_string]
        from azure.storage.fileshare import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples2")
        # [END create_share_client_from_conn_string]

        # Create the share
        share.create_share()

        try:
            # [START set_share_quota]
            # Set the quota for the share to 1GB
            share.set_share_quota(quota=1)
            # [END set_share_quota]

            # [START set_share_metadata]
            data = {'category': 'test'}
            share.set_share_metadata(metadata=data)
            # [END set_share_metadata]

            # Get the metadata for the share
            props = share.get_share_properties().metadata

        finally:
            # Delete the share
            share.delete_share()

    def set_share_properties(self):
        from azure.storage.fileshare import ShareClient
        share1 = ShareClient.from_connection_string(self.connection_string, "sharesamples3a")
        share2 = ShareClient.from_connection_string(self.connection_string, "sharesamples3b")

        # Create the share
        share1.create_share()
        share2.create_share()

        try:
            # [START set_share_properties]
            # Set the tier for the first share to Hot
            share1.set_share_properties(access_tier="Hot")
            # Set the quota for the first share to 3
            share1.set_share_properties(quota=3)
            # Set the tier for the second share to Cool and quota to 2
            share2.set_share_properties(access_tier=ShareAccessTier("Cool"), quota=2)

            # Get the shares' properties
            print(share1.get_share_properties().access_tier)
            print(share1.get_share_properties().quota)
            print(share2.get_share_properties().access_tier)
            print(share2.get_share_properties().quota)
            # [END set_share_properties]

        finally:
            # Delete the shares
            share1.delete_share()
            share2.delete_share()

    def list_directories_and_files(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples4")

        # Create the share
        share.create_share()

        try:
            # [START share_list_files_in_dir]
            # Create a directory in the share
            dir_client = share.create_directory("mydir")

            # Upload a file to the directory
            with open(SOURCE_FILE, "rb") as source_file:
                dir_client.upload_file(file_name="sample", data=source_file)

            # List files in the directory
            my_files = list(share.list_directories_and_files(directory_name="mydir"))
            print(my_files)
            # [END share_list_files_in_dir]
        finally:
            # Delete the share
            share.delete_share()

    def get_directory_or_file_client(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples5")

        # Get the directory client to interact with a specific directory
        my_dir = share.get_directory_client("dir1")

        # Get the file client to interact with a specific file
        my_file = share.get_file_client("dir1/myfile")

    def acquire_share_lease(self):
        # Instantiate the ShareClient from a connection string
        from azure.storage.fileshare import ShareClient
        share = ShareClient.from_connection_string(self.connection_string, "sharesamples")

        # Create the share
        share.create_share()

        # [START acquire_and_release_lease_on_share]
        share.create_directory("mydir")
        lease = share.acquire_lease()
        share.get_share_properties(lease=lease)
        share.delete_share(lease=lease)
        # [END acquire_and_release_lease_on_share]


if __name__ == '__main__':
    sample = ShareSamples()
    sample.create_share_snapshot()
    sample.set_share_quota_and_metadata()
    sample.set_share_properties()
    sample.list_directories_and_files()
    sample.get_directory_or_file_client()
    sample.acquire_share_lease()
